﻿local RSwapper3 = RSwapper3_E1115F90_2428_4DA0_AA34_4CCFCC5BFA00;

RSwapper3.ENABLED = "|cff00ff00[" .. RSwapper3.Locale.Enabled .. "]|r";
RSwapper3.DISABLED = "|cffff0000[" .. RSwapper3.Locale.Disabled .. "]|r";
RSwapper3.ISFISHING = "|cff00ffff[" .. RSwapper3.Locale.Fishing .. "]|r";
RSwapper3.ISJOUSTING = "|cffabcffc[" .. (GetSpellInfo(45105)) .. "]|r";

local ldb = LibStub:GetLibrary("LibDataBroker-1.1");
RSwapper3.DataObj = ldb:NewDataObject("RSwapper3", {type = "data source", icon = "Interface\\Icons\\Ability_Rogue_CheatDeath", text = RSwapper3.DISABLED, label = "RSwapper3", tocname = "RSwapper3"});

local dataobj = RSwapper3.DataObj;

function dataobj:UpdateText()
    local icon;

    if (not RSwapper3.Vars.Enable) then
        icon = "Interface\\Icons\\Ability_Rogue_FeignDeath";
        dataobj.text = RSwapper3.DISABLED;
    elseif (RSwapper3.Mode == 1 and RSwapper3.Status == 0) then
        if (RSwapper3.Vars.StealthSet) then
            dataobj.text = format(RSwapper3.HAVEDAGGER_FMT, RSwapper3.Vars.StealthSet);
        else
            dataobj.text = format(RSwapper3.HAVEDAGGER_FMT, RSwapper3.Locale.NotSet);
        end
    elseif (RSwapper3.Mode == 2 and RSwapper3.Status == 0) then
        if (RSwapper3.Vars.CombatSet) then
            dataobj.text = format(RSwapper3.HAVESWORD_FMT, RSwapper3.Vars.CombatSet);
        else
            dataobj.text = format(RSwapper3.HAVESWORD_FMT, RSwapper3.Locale.NotSet);
        end
    elseif (RSwapper3.Mode == 1) then
        icon = "Interface\\Icons\\Ability_Stealth";
        if (RSwapper3.Vars.StealthSet) then
            dataobj.text = format(RSwapper3.NEEDDAGGER_FMT, RSwapper3.Vars.StealthSet);
        else
            dataobj.text = format(RSwapper3.NEEDDAGGER_FMT, RSwapper3.Locale.NotSet);
        end
    elseif (RSwapper3.Mode == 2) then
        icon = "Interface\\Icons\\Spell_Nature_Invisibilty";
        if (RSwapper3.Vars.CombatSet) then
            dataobj.text = format(RSwapper3.NEEDSWORD_FMT, RSwapper3.Vars.CombatSet);
        else
            dataobj.text = format(RSwapper3.NEEDSWORD_FMT, RSwapper3.Locale.NotSet);
        end
    elseif (RSwapper3.Mode == 3) then
        dataobj.text = RSwapper3.ISFISHING;
    elseif (RSwapper3.Mode == 4) then
        dataobj.text = RSwapper3.ISJOUSTING;
    else
        icon = "Interface\\Icons\\Ability_Rogue_CheatDeath";
        dataobj.text = RSwapper3.ENABLED; -- it's in some state, but enabled.
    end

    if (not icon) then -- if we didn't define an icon, then we want the current weapon
        icon = GetInventoryItemTexture("player", RSwapper3.nMAINHANDSLOT)
            or "Interface\\Icons\\Ability_Rogue_CheatDeath";
    end

    dataobj.icon = icon;
end

function dataobj:OnTooltipShow()
    self:AddLine("RSwapper3");
    self:AddLine(" ");

    if (RSwapper3.NotRogue) then
        self:AddLine(RSwapper3.Locale.IsNotRogue);
        return;
    end

    if (not RSwapper3.Vars.Enable) then
        self:AddLine("|cffff0000" .. RSwapper3.Locale.Disabled .. "|r");
        return;
    end

    self:AddLine(dataobj.text);
    if (RSwapper3.Status == 1) then
        self:AddLine("|cffffff00" .. RSwapper3.Locale.Pending .. "|r");
    elseif (RSwapper3.Status == 2) then
        self:AddLine("|cffff0000" .. RSwapper3.Locale.NotSet .. "|r");
    elseif (RSwapper3.Status == 3) then
        self:AddLine("|cffff0000" .. RSwapper3.Locale.MissingPieces .. "|r");
    elseif (RSwapper3.Status == 4) then
        self:AddLine("|cffff0000" .. EQUIPMENT_MANAGER_IS_DISABLED .. "|r");
    end
end

function dataobj:OnEnter()
    GameTooltip:SetOwner(self, "ANCHOR_NONE");
    GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT");
    GameTooltip:ClearLines();
    dataobj.OnTooltipShow(GameTooltip);
    GameTooltip:Show();
end

function dataobj:OnLeave()
    GameTooltip:Hide();
end

function dataobj:OnClick(button)
    if (RSwapper3.NotRogue) then
        return;
    end
    GameTooltip:Hide();
    ToggleDropDownMenu(1, nil, dataobj.ContextMenu, self, 0, 0)
end

local menuitem = {};

dataobj.ContextMenu = CreateFrame("Frame", "RSwapper3DOContextMenu");
dataobj.ContextMenu.displayMode = "MENU";

function dataobj.ContextMenu:initialize(level)
    if (not level) then
        return;
    end

    wipe(menuitem);

    if (level == 1) then
        menuitem.checked = RSwapper3.Vars.Enable;
        menuitem.text = ENABLE;
        menuitem.func = RSwapper3.SetEnable;
        UIDropDownMenu_AddButton(menuitem, level);

        menuitem.checked = RSwapper3.Vars.Sound;
        menuitem.text = ENABLE_SOUNDFX;
        menuitem.func = RSwapper3.SetSound;
        UIDropDownMenu_AddButton(menuitem, level);

        menuitem.notCheckable = 1;
        menuitem.func = nil;
        menuitem.checked = nil;
        menuitem.hasArrow = true;
        menuitem.text = RSwapper3.Locale.StealthSet;
        menuitem.value = "StealthSet";
        UIDropDownMenu_AddButton(menuitem, level);

        menuitem.text = RSwapper3.Locale.CombatSet;
        menuitem.value = "CombatSet";
        UIDropDownMenu_AddButton(menuitem, level);

        menuitem.notCheckable = false;
        menuitem.hasArrow = false;
        menuitem.value = false;
        menuitem.checked = RSwapper3.Vars.Stuns;
        menuitem.text = RSwapper3.Locale.Stuns;
        menuitem.func = RSwapper3.GUI.MainFrame.OnClickStuns;
        menuitem.arg1 = "LeftButton";
        UIDropDownMenu_AddButton(menuitem, level);

        menuitem.checked = RSwapper3.Vars.Dance;
        menuitem.text = RSwapper3.Locale.Dance;
        menuitem.func = RSwapper3.GUI.MainFrame.OnClickDance;
        menuitem.arg1 = "LeftButton";
        UIDropDownMenu_AddButton(menuitem, level);
    end

    if (level == 2) then
        local value = UIDROPDOWNMENU_MENU_VALUE;
        local pickset = RSwapper3SetMenu.PickSet; -- no sense in rewriting the function
        local curvalue = RSwapper3.Vars[value];

        if (value ~= "StealthSet" and value ~= "CombatSet") then
            return;
        end

        RSwapper3SetMenu.Var = value;

        menuitem.checked = not (curvalue and GetEquipmentSetInfoByName(curvalue)); -- Is it 'Not Set' ?
        menuitem.text = RSwapper3.Locale.NotSet;
        menuitem.arg1 = nil;
        menuitem.func = pickset;
        UIDropDownMenu_AddButton(menuitem, level);

        for i = 1, GetNumEquipmentSets() do
            local name = GetEquipmentSetInfo(i);
            if (name) then
                menuitem.checked = name == curvalue;
                menuitem.text = name;
                menuitem.arg1 = name;
                menuitem.func = pickset;
                UIDropDownMenu_AddButton(menuitem, level);
            end
        end
    end
end

