﻿-- RSwapper 3: Swaps main-hand weapon for a rogue, depending on stealth, stuns and other buffs
--
-- Based on the rswapper code by Shambarimen (Muradin), who rewrote the code
-- for the original idea of RogueSwapper by Krax (Daggerspine) aka m0rfin.
--
-- This mod depends on certain Ace-3 libraries (included)
-- Other than that, this mod is self-sufficient (no other depends)

local RSwapper3 = RSwapper3_E1115F90_2428_4DA0_AA34_4CCFCC5BFA00;

local MainFrame = RSwapper3.GUI.MainFrame;
local dataobj = RSwapper3.DataObj;

local TargetStuns = RSwapper3.TargetStuns;
local PlayerStuns = RSwapper3.PlayerStuns;
local Sounds = RSwapper3.Sounds;

-- RSwapper3.Mode - Reference List
-- 1 Dagger Mode
-- 2 Sword Mode
-- 3 Is Fishing
-- 4 Is Jousting

-- RSwapper3.Status - Reference List
-- -1 Equipment Sets Changed
-- 1 Equipment Set is Pending Equip
-- 2 Equipment Set Not Defined
-- 3 Equipment Set Missing Pieces
-- 4 Equipment Mgr Not Enabled -- No longer applicable, since 4.0.1 you can't disable Eq Mgr, TODO: Eventually remove all references from code

-- Main Functions

--profileKeys - check Var table for this, clear Var table if this exists;

local MainFrame = RSwapper3.GUI.MainFrame;
MainFrame:RegisterEvent("PLAYER_LOGIN");

SlashCmdList['RSwapper3_GUI'] = MainFrame.Toggle;

_G['SLASH_RSwapper3_GUI1'] = '/rswapper';
_G['SLASH_RSwapper3_GUI2'] = '/rswapper3';
_G['SLASH_RSwapper3_GUI3'] = '/rswapperg';
_G['SLASH_RSwapper3_GUI4'] = '/rs3g';

function RSwapper3:PLAYER_LOGIN()
    if (select(2, UnitClass("player")) == "ROGUE") then
        print("|cffff00ffRSwapper3:|r", RSwapper3.Locale.IsRogue);
    else
        print("|cffff00ffRSwapper3:|r", RSwapper3.Locale.IsNotRogue);
        RSwapper3.NotRogue = true;
        return;
    end

    if (not RSwapper3DB) then
        RSwapper3DB = {};
    elseif (RSwapper3DB.profileKeys) then -- old vars
        wipe(RSwapper3DB);
    end

    RSwapper3.CharName = (UnitName("player")) .. " - " .. (GetRealmName());

    if (not RSwapper3DB[RSwapper3.CharName]) then
        RSwapper3DB[RSwapper3.CharName] = {};
    end
    RSwapper3.Vars = RSwapper3DB[RSwapper3.CharName];

    if (RSwapper3.Vars.Enable == nil) then
        RSwapper3.Vars.Enable = true;
    end
    if (RSwapper3.Vars.Sound == nil) then
        RSwapper3.Vars.Sound = true;
    end

    MainFrame:DoInit();

    MainFrame:RegisterEvent("UNIT_AURA");
    MainFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");
    MainFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
    MainFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED"); -- Used to SwapIfNeeded while in combat when if it originally failed

    dataobj:UpdateText();
    RSwapper3:SwapIfNeeded();
end

function RSwapper3:SetEnable()
    RSwapper3.Vars.Enable = not RSwapper3.Vars.Enable;
    if (RSwapper3.Vars.Enable) then
        if (RSwapper3.Vars.Sound) then PlaySoundFile(Sounds.Enable); end
        RSwapper3:SwapIfNeeded();
    else
        if (RSwapper3.Vars.Sound) then PlaySoundFile(Sounds.Disable); end
    end
    MainFrame.iddEnableBtn:SetChecked(RSwapper3.Vars.Enable);
    dataobj:UpdateText();
end

function RSwapper3:SetSound()
    RSwapper3.Vars.Sound = not RSwapper3.Vars.Sound;
    if (RSwapper3.Vars.Sound) then
        PlaySoundFile(Sounds.Sound);
    end
    MainFrame.iddSoundBtn:SetChecked(RSwapper3.Vars.Sound);
end

function RSwapper3:HasDebuff(unit, debuff_list)
    local i, cur_debuff = 1, nil;

    cur_debuff = UnitDebuff(unit, i);
    while (cur_debuff) do
        if (debuff_list[cur_debuff]) then
            return true;
        end
        i = i + 1;
        cur_debuff = UnitDebuff(unit, i);
    end
    return false;
end

local EqLoc = {};
local LastMH, LastOH;
function RSwapper3:SwapIfNeeded()
    if (not RSwapper3.Vars.Enable or GearManagerDialog:IsShown()) then return; end -- don't try to swap when equipment manager is open, player is configing

    local mainid = GetInventoryItemID("player", RSwapper3.nMAINHANDSLOT);
    if (mainid) then
        if (mainid == 46069 or mainid == 46070 or mainid == 46106) then
            RSwapper3.Mode = 4; -- Has a lance equipped, don't do anything
            RSwapper3.Status = 0;
            dataobj:UpdateText();
            return;
        end
        local itemsubtype = select(7, GetItemInfo(mainid));
        if (itemsubtype == RSwapper3.Locale.FishingPoles) then
            RSwapper3.Mode = 3; -- Has a fishing pole equipped, don't do anything
            RSwapper3.Status = 0;
            dataobj:UpdateText();
            return;
        end
    end

    local curset;
    local form = GetShapeshiftForm();

    if (RSwapper3.Vars.Stuns and RSwapper3:HasDebuff("target", RSwapper3.TargetStuns)) then
        form = 1; -- Pretend we're stealthed
    end

    local old_mode = RSwapper3.Mode;
    if (form == 1 or (RSwapper3.Vars.Dance and form == 3)) then -- check 1, 2 = stealth, 3 = shadow dance?
        RSwapper3.Mode = 1;
        curset = RSwapper3.Vars.StealthSet;
    else
        RSwapper3.Mode = 2;
        curset = RSwapper3.Vars.CombatSet;
    end

    local offid = GetInventoryItemID("player", RSwapper3.nSECONDARYHANDSLOT);
    if (old_mode == RSwapper3.Mode and RSwapper3.Status == 0 and LastMH == mainid and LastOH == offid) then
        return; -- Nothing's changed don't do a bunch of unnecessary processing
    end

--[[ As of patch 4.0.1 you can no longer disable Blizzard's Equipment Manager
    if (GetCVar("equipmentManager") ~= "1") then -- Make sure we determine what set we want before we decide we can't swap
        if (RSwapper3.Status == 4 and RSwapper3.TryAgain > GetTime()) then
            return;
        end
        RSwapper3.Status = 4;
        RSwapper3.TryAgain = GetTime() + 5;
        UIErrorsFrame:AddMessage("RSwapper3: " .. EQUIPMENT_MANAGER_IS_DISABLED, 1.0, 0.1, 0.1, 1.0);
        if (RSwapper3.Vars.Sound) then PlaySoundFile(Sounds.Error); end
        dataobj:UpdateText();
        return;
    end
  ]]

    if (not curset or not GetEquipmentSetInfoByName(curset)) then
        if (RSwapper3.Status == 2 and RSwapper3.TryAgain > GetTime()) then
            return;
        end
        RSwapper3.Status = 2; -- not defined
        RSwapper3.TryAgain = GetTime() + 5;
        if (RSwapper3.Mode == 1) then
            UIErrorsFrame:AddMessage(RSwapper3.Locale.StealthNotDefined, 1.0, 0.1, 0.1, 1.0);
        else
            UIErrorsFrame:AddMessage(RSwapper3.Locale.CombatNotDefined, 1.0, 0.1, 0.1, 1.0);
        end
        if (RSwapper3.Vars.Sound) then PlaySoundFile(Sounds.Error); end
        dataobj:UpdateText();
        return;
    end

    GetEquipmentSetLocations(curset, EqLoc);

    local mainloc = EqLoc[RSwapper3.nMAINHANDSLOT];
    local offloc = EqLoc[RSwapper3.nSECONDARYHANDSLOT];
    local player, bank, bags, slot, missingitems, needswap;

    if (mainloc == 0) then
        needswap = mainid; -- if there currently is a main hand we need to swap it to bags
    elseif (mainloc ~= 1) then
        player, bank, bags, slot = EquipmentManager_UnpackLocation(mainloc);
        if (not player or bank or bags or slot ~= RSwapper3.nMAINHANDSLOT) then
            needswap = true; -- make sure we try to swap regardless of if we have the weapon, to produce an error
        end
        if (not player and not bank and not bags) then
            missingitems = true; -- this is only true if we don't have the weapon
        end
    end

    if (offloc == 0) then
        needswap = offid;
    elseif (offloc ~= 1) then
        player, bank, bags, slot = EquipmentManager_UnpackLocation(offloc);
        if (not player or bank or bags or slot ~= RSwapper3.nSECONDARYHANDSLOT) then
            needswap = true; -- make sure we try to swap regardless of if we have the weapon, to produce an error
        end
        if (not player and not bank and not bags) then
            missingitems = true; -- this is only true if we don't have the weapon
        end
    end

    if (needswap) then
        if (RSwapper3.Mode == old_mode and RSwapper3.Status == 1 and GetTime() < RSwapper3.TryAgain) then
            return;
        end
        if (not missingitems or RSwapper3.Status ~= 3 or GetTime() > RSwapper3.TryAgain) then
            RSwapper3.Status = 1; -- Pending
            RSwapper3.TryAgain = GetTime() + 1; -- Wait a second before re-requesting the same swap.
            dataobj:UpdateText();
            UseEquipmentSet(curset); -- This will show the user an error if an item can't be found
        end
    else
        if (RSwapper3.Status == 1 and RSwapper3.Vars.Sound) then
            PlaySoundFile(Sounds.Swap);
        end
        LastMH, LastOH = mainid, offid;
        RSwapper3.Status = 0; -- ok
        dataobj:UpdateText();
    end

    if (RSwapper3.Status ~= 3 and missingitems) then
        RSwapper3.Status = 3; -- Missing pieces
        RSwapper3.TryAgain = GetTime() + 5;
        -- WoW has already displayed the error message, just play the sound
        if (RSwapper3.Vars.Sound) then PlaySoundFile(Sounds.Error); end
        dataobj:UpdateText();
    end
end

function RSwapper3:PLAYER_EQUIPMENT_CHANGED()
    RSwapper3:SwapIfNeeded();
end

function RSwapper3:UNIT_AURA(unitid)
    if (unitid == "player" or (unitid == "target" and RSwapper3.Vars.Stuns)) then
        RSwapper3:SwapIfNeeded();
    end
end

function RSwapper3:PLAYER_TARGET_CHANGED()
    if (RSwapper3.Vars.Stuns) then
        RSwapper3:SwapIfNeeded();
    end
end

function RSwapper3:UNIT_SPELLCAST_SUCCEEDED()
    RSwapper3:SwapIfNeeded();
end

